@@ -1,11 +1,11 @@
 package Spreadsheet::HTML;
 use strict;
 use warnings FATAL => 'all';
-our $VERSION = '0.05';
+our $VERSION = '0.06';
 
 use Data::Dumper;
-use HTML::Entities;
 use Math::Matrix;
+use HTML::Element;
 
 sub new {
     my $class = shift;
@@ -13,61 +13,67 @@ sub new {
     return bless { %attrs }, $class;
 }
 
-sub generate { _make_table( _data( @_ ) ) }
+sub generate    { _wrapper( sub { @_ }, @_ ) }
+sub transpose   { _wrapper( sub { @{ Math::Matrix::transpose( [@_] ) } }, @_ ) }
+sub reverse     { _wrapper( sub { reverse @_ }, @_ ) }
 
-sub transpose { _make_table( @{ Math::Matrix::transpose( [ _data( @_ ) ] ) } ) }
+sub _wrapper {
+    my $sub   = shift;
+    my $self  = shift if UNIVERSAL::isa( $_[0], __PACKAGE__ );
+    my $attrs = ref($_[0]) eq 'HASH' ? shift : {};
 
-sub reverse { _make_table( reverse _data( @_ ) ) }
+    if (@_ > 1 && defined($_[0]) && !ref($_[0]) ) {
+        my %args = @_;
+        @_ = delete $args{data};
+        $attrs = {%args};    
+    }
+
+    my @data  = $self ? $self->process_data( @_ ) : process_data( @_ );
+    return _make_table( $attrs, $sub->( @data ) );
+}
 
 sub process_data {
     my ($self, @data);
 
 	if (UNIVERSAL::isa( $_[0], __PACKAGE__ )) {
-        # called as method
         $self = shift;
-        return @{ $self->{data} } if $self->{__processed_data__};
-        @data = @{ ref($self->{data}) ? $self->{data} : [[ $self->{data} ]] };
-    } else {
-        # called as function
-        @data = @_ > 1 ? @_ : @{ ref($_[0]) ? $_[0] : [[ $_[0] ]] };
+        return @{ $self->{data} } if exists $self->{__processed_data__};
+        $self->{table_attrs} = shift if ref($_[0]) eq 'HASH';
+        @_ = $self->{data};
     }
 
+    @data = @_ > 1 ? @_ : @{ ref($_[0]) ? $_[0] : [ $_[0] ] };
     @data = [ @data ] unless ref( $data[0] ) eq 'ARRAY';
+    @data = [ undef ] unless @{ $data[0] };
 
-    $self->{data} = _encode( @data );
+    $self->{data} = _process( @data );
     $self->{data} = _mark_headers( $self->{data} );
     $self->{__processed_data__} = 1;
     return @{ $self->{data} };
 }
 
-sub _data {
-    my ($self, @data);
-	if (UNIVERSAL::isa( $_[0], __PACKAGE__ )) {
-        $self = shift;
-        @data = $self->process_data;
-    } else {
-        @data = process_data( @_ );
-    }
-    return @data;
-}
-
 sub _make_table {
-    my $rows = '';
-    for my $row (@_) {
-        $rows .= '<tr>';
-        for my $cell (@$row) {
-            if (ref($cell)) {
-                $rows .= sprintf( '<th>%s</th>', $cell->[0] );
-            } else {
-                $rows .= sprintf( '<td>%s</td>', $cell );
-            }
-        }
-        $rows .= '</tr>';
-    }
+    my $attrs = ref($_[0]) eq 'HASH' ? shift : {};
+    $attrs->{$_} ||= {} for qw( table tr th td );
 
-    return "<table>$rows</table>";
+    my $encodes = delete $attrs->{encodes} || '';
+    my $indent  = delete $attrs->{indent};
+
+    my $table = HTML::Element->new_from_lol(
+        [table => $attrs->{table},
+            map [tr => $attrs->{tr},
+                map ref($_) 
+                    ? [ th => $attrs->{td}, @$_ ]
+                    : [ td => $attrs->{td}, $_ ], @$_
+            ], @_
+        ],
+    );
+
+    chomp( my $html = $table->as_HTML( $encodes, $indent ) );
+    return $html;
 }
 
+# just a way to seperate th cells from td cells now (for later)
 sub _mark_headers {
     my $data = shift;
     $data->[0] = [ map [$_], @{ $data->[0] } ];
@@ -75,11 +81,11 @@ sub _mark_headers {
 }
 
 
-sub _encode {
+sub _process {
     my @data = @_;
 
     # padding is determined by first row (the header)
-    my $max_cols = scalar @{ $data[0] || [] };
+    my $max_cols = scalar @{ $data[0] };
     for my $row (@data[1 .. $#data]) {
         push @$row, undef for 1 .. ($max_cols - scalar @$row);
     }
@@ -88,8 +94,6 @@ sub _encode {
         map { [
             map {
                 do { no warnings; s/^\s*$/&nbsp;/g };
-                decode_entities( $_ );    
-                encode_entities( $_ );    
                 s/\n/<br \/>/g;
                 $_
             } @$_
@@ -144,6 +148,11 @@ Used to transpose data from portrait to landscape.
 
 =item new()
 
+  my $table = Spreadsheet::HTML->new;
+  my $table = Spreadsheet::HTML->new( @data );
+  my $table = Spreadsheet::HTML->new( $data );
+  my $table = Spreadsheet::HTML->new( data => $data );
+
 Constructs object. Currently accepts one parameter: data.
 Data should be a two dimensional array and you should 
 expect the first row to be treated as the header (which
@@ -152,6 +161,20 @@ of <td> tags).
 
 =item generate()
 
+  my $html = $table->generate;
+  my $html = $table->generate( indent => '    ' );
+  my $html = $table->generate( encode => '<>&=' );
+  my $html = $table->generate( table => { class => 'foo' } );
+
+  my $html = Spreadsheet::HTML::generate( @data );
+  my $html = Spreadsheet::HTML::generate( $data );
+  my $html = Spreadsheet::HTML::generate(
+      data   => $data,
+      indent => '    ',
+      encode => '<>&=',
+      table  => { class => 'foo' },
+  );
+
 Returns a string that contains the rendered HTML table.
 Currently (and subject to change if better ideas arise),
 all data will:
@@ -160,32 +183,43 @@ all data will:
 
 =item - be converted to &nbsp; if empty
 
-=item - be HTML entity decoded
-
-=item - be HTML entity encoded
-
 =item - have any newlines converted to <br> tags
 
 =back
 
-The first row of the data will have columns wrapped with
-<tr> tags and the remaining rows will be wraped with 
-<td> tags.
-
 =item transpose()
 
+  (same usage as generate)    
+
 Uses Math::Matrix to rotate the data 90 degrees and
 then returns a string containing the rendered HTML table.
 
 =item reverse()
 
+  (same usage as generate)    
+
 Uses Math::Matrix to rotate the data 180 degrees and
 then returns a string containing the rendered HTML table.
 
 =item process_data()
 
 Returns the munged data before it is used to generate
-a rendered HTML table.
+a rendered HTML table. This is called on your behalf, but
+is available should you want it.
+
+=back
+
+=head1 REQUIRES
+
+=over 4
+
+=item L<HTML::Tree>
+
+Used to generate HTML.
+
+=item L<Math::Matrix>
+
+Used for transposing data.
 
 =back
 
